/*****************************************************************************************
 * rmb_rpt_portpie_chart.sas  
 *
 *  Copyright (C) 2010, SAS Institute Inc., Cary, NC, USA. All Rights Reserved     
 *
 * REPORT NAME: PORTFOLIO PIE CHART (ONLY IF ALTYPE AND PRODUCT TYPE are present in the input data.
 *
 * PURPOSE: 
 *              
 * NOTES: 
 *
 *****************************************************************************************/

%macro rmba_rpt_portpie_chart(in_ds=mtm_final, in_sum_var_nm=mark_to_market_amt, in_oth_pct=1);

   
   /*Check if input dataset exists*/
   %if (%rsk_dsexist(&in_ds.)) %then %do;
   
      /*Check if there is data*/
      %if (%rsk_attrn(&in_ds., NOBS) ne 0) %then %do;   
      
         /*Determine if the input data has the required ccvs of ALTYPE and PRODUCT_TYPE*/
         %if %rsk_varexist(&in_ds, ALTYPE) AND %rsk_varexist(&in_ds, PRODUCT_TYPE) %then %do;
         
            /*Divide the data into assets, liabilities and total amount*/
            
            data portf_assets portf_liab;
               set &in_ds;
               
               if PRODUCT_TYPE ne "+" then do;
               
                  if ALTYPE="ASSET" then do;
                  
                     if &in_sum_var_nm >= 0 then do;
                        output portf_assets;
                     end;
                     else do;
                        output portf_liab;
                     end;
                     
                  end;
                  
                  if ALTYPE="LIABILITY" then do;
                  
                     if &in_sum_var_nm <= 0 then do;
                        output portf_liab;
                     end;
                     else do;
                        output portf_assets;
                     end;
                  
                  end;
                  
              end; 
            run;
            
            /*For pie chart graphing purposes. Liab are reported as negative, but gchart can only handle non-negative value*/
            
            data portf_liab;
               set portf_liab;
               &in_sum_var_nm=abs(&in_sum_var_nm.);
            run;
               
         
            /*Generate each pie chart if data is available*/
         
             %if %rsk_attrn(portf_assets, nobs) > 0 %then %do;
             
                %_rmba_generate_pie_chart (in_ds=portf_assets, sum_stat=PRODUCT_TYPE, sum_var=&in_sum_var_nm, title_msg_key=rmb_rpt_portfolio_total_assets, oth_pct=&in_oth_pct);
                
             %end;
             
             %if %rsk_attrn(portf_liab, nobs) > 0 %then %do;
             
                %_rmba_generate_pie_chart (in_ds=portf_liab, sum_stat=PRODUCT_TYPE, sum_var=&in_sum_var_nm, title_msg_key=rmb_rpt_portfolio_total_liab, oth_pct=&in_oth_pct);
             
             %end;
             
             /*
             %if %rsk_attrn(portf_total, nobs) > 0 %then %do;
              
                %_rmba_generate_pie_chart (in_ds=portf_total, sum_stat=PRODUCT_TYPE, sum_var=&in_sum_var_nm, title_msg_key=rmb_rpt_portfolio_total, oth_pct=&in_oth_pct);
             
             %end;
             */
             
      
         %end;
         %else %do; /*The input data does NOT have the required ccvs.*/
         
               title1 "%rsk_get_label(rmb_rpt_subtitle_portfolio_bal_sum)";      
              
               data no_ccv_message;
                   length message $1000;
                   message="%rsk_get_label(rsk_rpt_no_ccv_data_for_pie_chart)";
                   output;
                run;
   
                proc print data=no_ccv_message noobs;
                run;
                
               title1;
         
         %end;
      
      %end; /*Check if ds has observations*/   
      %else %do;
       
         title1 "%rsk_get_label(rmb_rpt_subtitle_portfolio_bal_sum)";      

         data no_ccv_message;
            length message $1000;
            message="%rsk_get_label(rsk_rpt_no_port_summary_data)";
            output;
         run;
   
         proc print data=no_ccv_message noobs;
         run;
                
         title1;  
      
      
      %end;   
      
   %end; /*Check if ds exists and has data*/
   %else %do;
   
      title1 "%rsk_get_label(rmb_rpt_subtitle_portfolio_bal_sum)";      
              
     data no_ccv_message;
        length message $1000;
        message="%rsk_get_label(rsk_rpt_no_port_summary_data)";
        output;
     run;
   
     proc print data=no_ccv_message noobs;
     run;
                
      title1;      
   
   %end;
   
   /*Reset to what it was in ods*/
   footnote1 height=1 j=center "-%rsk_get_label(RMB_rpt_%trim(&REPORT_CD.)_title)-" j=right "-^{thispage}-"; 
      
%mend;

%macro _rmba_generate_pie_chart (in_ds=, sum_stat=PRODUCT_TYPE, sum_var=mark_to_market_amt, title_msg_key=, oth_pct=1);

   /*Determine if we need to scale the pie chart data*/
   %local scale_pie_chart_flg;
   %let scale_pie_chart_flg=0; /*false*/
   
   /*Calculate total for display*/
   proc summary data=&in_ds;
      var &sum_var;                  
     output out=display_total
     sum=TOTAL;
   run;

   /*Grab the total amount*/
   data _NULL_;
      set display_total;
      call symput("total_amt", TOTAL);
      call symput("portf_total_amt", put(TOTAL, &use_currency_format.));
   run;
   
   data mag_scale_test;
      set &in_ds.;
      if &sum_var. ge 999999999999 then do;
         output;
      end;
   run;
   
   /*Determine if we need to scale the data first*/
   %if (&total_amt. ge 999999999)
   
       OR
       
       (%rsk_attrn(work.mag_scale_test, NOBS) > 0)
       
   %then %do;
     
         /*Scale the values*/
         %let scale_pie_chart_flg=1;

         /*Scale the data by millions*/
            
         data &in_ds.;
            set &in_ds.;
            &sum_var=&sum_var/1000000;
         run;
         
         /*Re-calculate total for display*/
         proc summary data=&in_ds;
            var &sum_var;                  
            output out=display_total
            sum=TOTAL;
         run;

         /*Grab the total amount*/
         data _NULL_;
            set display_total;
            call symput("total_amt", TOTAL);
            call symput("portf_total_amt", put(TOTAL, &use_currency_format.));
         run;
   
         
    %end;

   /*Determine what the percentage of the OTHER slice should be */
   data scale_test; 
      set &in_ds.;
      length percent_calc check 8.;
      percent_calc= (&sum_var. / &total_amt.) * 100 ;
      check=round(percent_calc, 0.1);
   run;
   
   data values_above values_below;
      set scale_test;
      if check > 0 then do;
         output values_above; 
      end;
      else do;
         output values_below;
      end;
   run;
   
   proc sql noprint;
      select min(round(check,1)) into: min_pct 
      from values_above;
   quit;
   
   %local percent_value_to_use;
     
   %if &min_pct <= &oth_pct %then %do;
   
      /*Use the specified percentage*/
      %let percent_value_to_use=&oth_pct;
      
   %end;
   %else %do;
   
      /*Check to see if any values in values_below */
      %if %rsk_attrn(values_below,NOBS) eq 0 %then %do; /*No zero entries, just use what was specified*/
      
         /*Use the specified percentage*/
         %let percent_value_to_use=&oth_pct;
      
      %end;
      %else %do; 
      
         /*Use the min_pct value so that we capture the min and not have a zero other slice*/
         %let percent_value_to_use=&min_pct;
         
      %end;
      
   %end;
   
   proc sort data=&in_ds.;
      by &sum_stat.;
   run;
   
   %local no_of_cat num_across_to_use;
   
   proc sql noprint;
      select count(distinct &sum_stat.) into: no_of_cat
      from values_above;
   quit;
   
   %if &no_of_cat < 3 %then %do;
   
      %let num_across_to_use=%eval(&no_of_cat + 1);
      
   %end;
   %else %do;
   
      %let num_across_to_use=4;
   
   %end; 
   
   
   goptions device=gif xpixels=600 ypixels=800;

   
   proc gchart data=&in_ds.;
      legend1 across=&num_across_to_use. frame label=("%rsk_get_label(RMB_RPT_&sum_stat.)") position=(center bottom) shape=bar(2,.5) ;
      pie &sum_stat. / noheading type=sum sumvar=&sum_var. 
                       other=&percent_value_to_use. 
                       percent=arrow value=arrow legend=legend1
                       ;
      format &sum_var. &use_currency_format.;
      
      title1 "%rsk_get_label(rmb_rpt_subtitle_portfolio_bal_sum)";      
      title2 "%rsk_get_label(&title_msg_key)";
      title3 "%superq(portf_total_amt)";
      
      %if &scale_pie_chart_flg %then %do;
      
         title4 "( %rsk_get_label(rmb_rpt_portfolio_pie_scale) )";
         
      %end;
      
      footnote1 height=1 j=center "-%rsk_get_label(RMB_rpt_%trim(&REPORT_CD.)_title)-"; 

   run;
   quit;
   
   ODS &REPORT_TYPE STARTPAGE=YES;
    
   title1;
   title2;
   title3;
   title4;
           
   
%mend;
